@@ -1,8 +1,4 @@
-//
-// Sysinfo
-//
-// Copyright (c) 2021 Guillaume Gomez
-//
+// Take a look at the license at the top of the repository in the LICENSE file.
 
 use std::ffi::CStr;
 use std::mem::{self, MaybeUninit};
@@ -13,7 +9,7 @@ use std::borrow::Borrow;
 
 use libc::{c_int, c_void, gid_t, kill, size_t, uid_t};
 
-use crate::{DiskUsage, Pid, ProcessExt, ProcessStatus, Signal};
+use crate::{DiskUsage, Pid, ProcessExt, ProcessRefreshKind, ProcessStatus, Signal};
 
 use crate::sys::process::ThreadStatus;
 use crate::sys::system::Wrap;
@@ -34,6 +30,7 @@ pub struct Process {
     old_utime: u64,
     old_stime: u64,
     start_time: u64,
+    run_time: u64,
     updated: bool,
     cpu_usage: f32,
     /// User id of the process owner.
@@ -70,6 +67,7 @@ impl Process {
             old_stime: 0,
             updated: true,
             start_time: 0,
+            run_time: 0,
             uid: 0,
             gid: 0,
             process_status: ProcessStatus::Unknown(0),
@@ -86,6 +84,7 @@ impl Process {
         pid: Pid,
         parent: Option<Pid>,
         start_time: u64,
+        run_time: u64,
         exe: PathBuf,
         name: String,
         cwd: PathBuf,
@@ -109,6 +108,7 @@ impl Process {
             old_stime: 0,
             updated: true,
             start_time,
+            run_time,
             uid: 0,
             gid: 0,
             process_status: ProcessStatus::Unknown(0),
@@ -119,10 +119,8 @@ impl Process {
             written_bytes: 0,
         }
     }
-}
 
-impl ProcessExt for Process {
-    fn new(pid: Pid, parent: Option<Pid>, start_time: u64) -> Process {
+    pub(crate) fn new(pid: Pid, parent: Option<Pid>, start_time: u64, run_time: u64) -> Process {
         Process {
             name: String::new(),
             pid,
@@ -139,6 +137,7 @@ impl ProcessExt for Process {
             old_stime: 0,
             updated: true,
             start_time,
+            run_time,
             uid: 0,
             gid: 0,
             process_status: ProcessStatus::Unknown(0),
@@ -149,8 +148,14 @@ impl ProcessExt for Process {
             written_bytes: 0,
         }
     }
+}
 
-    fn kill(&self, signal: Signal) -> bool {
+impl ProcessExt for Process {
+    fn kill(&self) -> bool {
+        self.kill_with(Signal::Kill).unwrap()
+    }
+
+    fn kill_with(&self, signal: Signal) -> Option<bool> {
         let c_signal = match signal {
             Signal::Hangup => libc::SIGHUP,
             Signal::Interrupt => libc::SIGINT,
@@ -184,10 +189,10 @@ impl ProcessExt for Process {
             // SIGPOLL doesn't exist on apple targets but since it's an equivalent of SIGIO on unix,
             // we simply use the SIGIO constant.
             Signal::Poll => libc::SIGIO,
-            Signal::Power => return false,
+            Signal::Power => return None,
             Signal::Sys => libc::SIGSYS,
         };
-        unsafe { kill(self.pid, c_signal) == 0 }
+        unsafe { Some(kill(self.pid, c_signal) == 0) }
     }
 
     fn name(&self) -> &str {
@@ -238,6 +243,10 @@ impl ProcessExt for Process {
         self.start_time
     }
 
+    fn run_time(&self) -> u64 {
+        self.run_time
+    }
+
     fn cpu_usage(&self) -> f32 {
         self.cpu_usage
     }
@@ -336,8 +345,9 @@ pub(crate) fn update_process(
     pid: Pid,
     mut size: size_t,
     time_interval: Option<f64>,
+    now: u64,
+    refresh_kind: ProcessRefreshKind,
 ) -> Result<Option<Process>, ()> {
-    let mut mib: [c_int; 3] = [libc::CTL_KERN, libc::KERN_ARGMAX, 0];
     let mut proc_args = Vec::with_capacity(size as usize);
 
     unsafe {
@@ -366,11 +376,15 @@ pub(crate) fn update_process(
                 (0, 0, None)
             };
             p.status = thread_status;
-            compute_cpu_usage(p, task_info, system_time, user_time, time_interval);
+            if refresh_kind.cpu() {
+                compute_cpu_usage(p, task_info, system_time, user_time, time_interval);
+            }
 
             p.memory = task_info.pti_resident_size / 1_000;
             p.virtual_memory = task_info.pti_virtual_size / 1_000;
-            update_proc_disk_activity(p);
+            if refresh_kind.disk_usage() {
+                update_proc_disk_activity(p);
+            }
             return Ok(None);
         }
 
@@ -429,9 +443,7 @@ pub(crate) fn update_process(
         };
 
         let ptr: *mut u8 = proc_args.as_mut_slice().as_mut_ptr();
-        mib[0] = libc::CTL_KERN;
-        mib[1] = libc::KERN_PROCARGS2;
-        mib[2] = pid as c_int;
+        let mut mib = [libc::CTL_KERN, libc::KERN_PROCARGS2, pid as c_int];
         /*
          * /---------------\ 0x00000000
          * | ::::::::::::: |
@@ -464,7 +476,7 @@ pub(crate) fn update_process(
          */
         if libc::sysctl(
             mib.as_mut_ptr(),
-            3,
+            mib.len() as _,
             ptr as *mut c_void,
             &mut size,
             std::ptr::null_mut(),
@@ -483,6 +495,9 @@ pub(crate) fn update_process(
         let mut cp = ptr.add(mem::size_of::<c_int>());
         let mut start = cp;
 
+        let start_time = info.pbi_start_tvsec;
+        let run_time = now.saturating_sub(start_time);
+
         let mut p = if cp < ptr.add(size) {
             while cp < ptr.add(size) && *cp != 0 {
                 cp = cp.offset(1);
@@ -556,7 +571,8 @@ pub(crate) fn update_process(
             Process::new_with(
                 pid,
                 parent,
-                info.pbi_start_tvsec,
+                start_time,
+                run_time,
                 exe,
                 name,
                 cwd,
@@ -565,7 +581,7 @@ pub(crate) fn update_process(
                 root,
             )
         } else {
-            Process::new(pid, parent, info.pbi_start_tvsec)
+            Process::new(pid, parent, start_time, run_time)
         };
 
         let task_info = get_task_info(pid);
@@ -576,7 +592,9 @@ pub(crate) fn update_process(
         p.uid = info.pbi_uid;
         p.gid = info.pbi_gid;
         p.process_status = ProcessStatus::from(info.pbi_status);
-        update_proc_disk_activity(&mut p);
+        if refresh_kind.disk_usage() {
+            update_proc_disk_activity(&mut p);
+        }
         Ok(Some(p))
     }
 }
