@@ -1,8 +1,4 @@
-//
-// Sysinfo
-//
-// Copyright (c) 2015 Guillaume Gomez
-//
+// Take a look at the license at the top of the repository in the LICENSE file.
 
 use crate::sys::component::Component;
 use crate::sys::disk::*;
@@ -13,7 +9,7 @@ use crate::sys::processor::*;
 #[cfg(target_os = "macos")]
 use core_foundation_sys::base::{kCFAllocatorDefault, CFRelease};
 
-use crate::{LoadAvg, Pid, ProcessorExt, RefreshKind, SystemExt, User};
+use crate::{LoadAvg, Pid, ProcessRefreshKind, ProcessorExt, RefreshKind, SystemExt, User};
 
 #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
 use crate::ProcessExt;
@@ -22,13 +18,14 @@ use std::cell::UnsafeCell;
 use std::collections::HashMap;
 use std::mem;
 use std::sync::Arc;
+use std::time::SystemTime;
 
 #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
 use libc::size_t;
 
 use libc::{
-    c_char, c_int, c_void, host_statistics64, mach_port_t, mach_task_self, sysconf, sysctl,
-    sysctlbyname, timeval, vm_statistics64, _SC_PAGESIZE,
+    c_char, c_int, c_void, host_statistics64, mach_port_t, sysconf, sysctl, sysctlbyname, timeval,
+    vm_statistics64, _SC_PAGESIZE,
 };
 
 #[doc = include_str!("../../md_doc/system.md")]
@@ -87,16 +84,8 @@ impl System {
     fn clear_procs(&mut self) {
         use crate::sys::macos::process;
 
-        let mut to_delete = Vec::new();
-
-        for (pid, proc_) in &mut self.process_list {
-            if !process::has_been_updated(proc_) {
-                to_delete.push(*pid);
-            }
-        }
-        for pid in to_delete {
-            self.process_list.remove(&pid);
-        }
+        self.process_list
+            .retain(|_, proc_| process::has_been_updated(proc_));
     }
 }
 
@@ -110,7 +99,7 @@ fn boot_time() -> u64 {
     if unsafe {
         sysctl(
             mib.as_mut_ptr(),
-            2,
+            mib.len() as _,
             &mut boot_time as *mut timeval as *mut _,
             &mut len,
             std::ptr::null_mut(),
@@ -124,12 +113,18 @@ fn boot_time() -> u64 {
     }
 }
 
+fn get_now() -> u64 {
+    SystemTime::now()
+        .duration_since(SystemTime::UNIX_EPOCH)
+        .map(|n| n.as_secs())
+        .unwrap_or(0)
+}
+
 impl SystemExt for System {
     const IS_SUPPORTED: bool = true;
 
     fn new_with_specifics(refreshes: RefreshKind) -> System {
         let port = unsafe { libc::mach_host_self() };
-        let (global_processor, processors) = init_processors(port);
 
         let mut s = System {
             process_list: HashMap::with_capacity(200),
@@ -138,8 +133,14 @@ impl SystemExt for System {
             mem_available: 0,
             swap_total: 0,
             swap_free: 0,
-            global_processor,
-            processors,
+            global_processor: Processor::new(
+                "0".to_owned(),
+                Arc::new(ProcessorData::new(std::ptr::null_mut(), 0)),
+                0,
+                String::new(),
+                String::new(),
+            ),
+            processors: Vec::new(),
             page_size_kb: unsafe { sysconf(_SC_PAGESIZE) as u64 / 1_000 },
             components: Vec::with_capacity(2),
             #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
@@ -238,29 +239,33 @@ impl SystemExt for System {
 
     fn refresh_cpu(&mut self) {
         let processors = &mut self.processors;
-        update_processor_usage(
-            self.port,
-            &mut self.global_processor,
-            |proc_data, cpu_info| {
-                let mut percentage = 0f32;
-                let mut offset = 0;
-                for proc_ in processors.iter_mut() {
-                    let cpu_usage = compute_processor_usage(proc_, cpu_info, offset);
-                    proc_.update(cpu_usage, Arc::clone(&proc_data));
-                    percentage += proc_.cpu_usage();
+        if processors.is_empty() {
+            init_processors(self.port, processors, &mut self.global_processor);
+        } else {
+            update_processor_usage(
+                self.port,
+                &mut self.global_processor,
+                |proc_data, cpu_info| {
+                    let mut percentage = 0f32;
+                    let mut offset = 0;
+                    for proc_ in processors.iter_mut() {
+                        let cpu_usage = compute_processor_usage(proc_, cpu_info, offset);
+                        proc_.update(cpu_usage, Arc::clone(&proc_data));
+                        percentage += proc_.cpu_usage();
 
-                    offset += libc::CPU_STATE_MAX as isize;
-                }
-                (percentage, processors.len())
-            },
-        );
+                        offset += libc::CPU_STATE_MAX as isize;
+                    }
+                    (percentage, processors.len())
+                },
+            );
+        }
     }
 
     #[cfg(any(target_os = "ios", feature = "apple-sandbox"))]
-    fn refresh_processes(&mut self) {}
+    fn refresh_processes_specifics(&mut self, _refresh_kind: ProcessRefreshKind) {}
 
     #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
-    fn refresh_processes(&mut self) {
+    fn refresh_processes_specifics(&mut self, refresh_kind: ProcessRefreshKind) {
         use crate::utils::into_iter;
 
         let count = unsafe { libc::proc_listallpids(::std::ptr::null_mut(), 0) };
@@ -268,6 +273,7 @@ impl SystemExt for System {
             return;
         }
         if let Some(pids) = get_proc_list() {
+            let now = get_now();
             let arg_max = get_arg_max();
             let port = self.port;
             let time_interval = self.clock_info.as_mut().map(|c| c.get_time_interval(port));
@@ -279,7 +285,14 @@ impl SystemExt for System {
 
                 into_iter(pids)
                     .flat_map(|pid| {
-                        match update_process(wrap, pid, arg_max as size_t, time_interval) {
+                        match update_process(
+                            wrap,
+                            pid,
+                            arg_max as size_t,
+                            time_interval,
+                            now,
+                            refresh_kind,
+                        ) {
                             Ok(x) => x,
                             _ => None,
                         }
@@ -294,18 +307,26 @@ impl SystemExt for System {
     }
 
     #[cfg(any(target_os = "ios", feature = "apple-sandbox"))]
-    fn refresh_process(&mut self, _: Pid) -> bool {
+    fn refresh_process_specifics(&mut self, _pid: Pid, _refresh_kind: ProcessRefreshKind) -> bool {
         false
     }
 
     #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
-    fn refresh_process(&mut self, pid: Pid) -> bool {
+    fn refresh_process_specifics(&mut self, pid: Pid, refresh_kind: ProcessRefreshKind) -> bool {
+        let now = get_now();
         let arg_max = get_arg_max();
         let port = self.port;
         let time_interval = self.clock_info.as_mut().map(|c| c.get_time_interval(port));
         match {
             let wrap = Wrap(UnsafeCell::new(&mut self.process_list));
-            update_process(&wrap, pid, arg_max as size_t, time_interval)
+            update_process(
+                &wrap,
+                pid,
+                arg_max as size_t,
+                time_interval,
+                now,
+                refresh_kind,
+            )
         } {
             Ok(Some(p)) => {
                 self.process_list.insert(p.pid(), p);
@@ -399,7 +420,7 @@ impl SystemExt for System {
         self.swap_free
     }
 
-    // need to be checked
+    // TODO: need to be checked
     fn used_swap(&self) -> u64 {
         self.swap_total - self.swap_free
     }
@@ -565,7 +586,7 @@ fn get_io_service_connection() -> Option<ffi::io_connect_t> {
         }
 
         let mut conn = 0;
-        let result = ffi::IOServiceOpen(device, mach_task_self(), 0, &mut conn);
+        let result = ffi::IOServiceOpen(device, libc::mach_task_self(), 0, &mut conn);
         ffi::IOObjectRelease(device);
         if result != ffi::KIO_RETURN_SUCCESS {
             sysinfo_debug!("Error: IOServiceOpen() = {}", result);
@@ -578,13 +599,13 @@ fn get_io_service_connection() -> Option<ffi::io_connect_t> {
 
 #[cfg(all(target_os = "macos", not(feature = "apple-sandbox")))]
 fn get_arg_max() -> usize {
-    let mut mib: [c_int; 3] = [libc::CTL_KERN, libc::KERN_ARGMAX, 0];
+    let mut mib = [libc::CTL_KERN, libc::KERN_ARGMAX];
     let mut arg_max = 0i32;
     let mut size = mem::size_of::<c_int>();
     unsafe {
         if sysctl(
             mib.as_mut_ptr(),
-            2,
+            mib.len() as _,
             (&mut arg_max) as *mut i32 as *mut c_void,
             &mut size,
             std::ptr::null_mut(),
@@ -609,7 +630,7 @@ pub(crate) unsafe fn get_sys_value(
     mib[1] = low as i32;
     sysctl(
         mib.as_mut_ptr(),
-        2,
+        mib.len() as _,
         value,
         &mut len as *mut usize,
         std::ptr::null_mut(),
@@ -635,7 +656,7 @@ fn get_system_info(value: c_int, default: Option<&str>) -> Option<String> {
     unsafe {
         sysctl(
             mib.as_mut_ptr(),
-            2,
+            mib.len() as _,
             std::ptr::null_mut(),
             &mut size,
             std::ptr::null_mut(),
@@ -653,7 +674,7 @@ fn get_system_info(value: c_int, default: Option<&str>) -> Option<String> {
         if unsafe {
             sysctl(
                 mib.as_mut_ptr(),
-                2,
+                mib.len() as _,
                 buf.as_mut_ptr() as _,
                 &mut size,
                 std::ptr::null_mut(),
