@@ -1,8 +1,4 @@
-//
-// Sysinfo
-//
-// Copyright (c) 2015 Guillaume Gomez
-//
+// Take a look at the license at the top of the repository in the LICENSE file.
 
 use std::cell::UnsafeCell;
 use std::collections::HashMap;
@@ -12,12 +8,12 @@ use std::io::Read;
 use std::path::{Path, PathBuf};
 use std::str::FromStr;
 
-use libc::{gid_t, kill, sysconf, uid_t, _SC_CLK_TCK};
+use libc::{gid_t, kill, uid_t};
 
 use crate::sys::system::REMAINING_FILES;
 use crate::sys::utils::{get_all_data, get_all_data_from_file, realpath};
 use crate::utils::into_iter;
-use crate::{DiskUsage, Pid, ProcessExt, ProcessStatus, Signal};
+use crate::{DiskUsage, Pid, ProcessExt, ProcessRefreshKind, ProcessStatus, Signal};
 
 #[doc(hidden)]
 impl From<u32> for ProcessStatus {
@@ -65,7 +61,7 @@ impl fmt::Display for ProcessStatus {
             ProcessStatus::Wakekill => "Wakekill",
             ProcessStatus::Waking => "Waking",
             ProcessStatus::Parked => "Parked",
-            ProcessStatus::Unknown(_) => "Unknown",
+            _ => "Unknown",
         })
     }
 }
@@ -87,7 +83,8 @@ pub struct Process {
     old_utime: u64,
     old_stime: u64,
     start_time: u64,
-    updated: bool,
+    run_time: u64,
+    pub(crate) updated: bool,
     cpu_usage: f32,
     /// User id of the process owner.
     pub uid: uid_t,
@@ -103,8 +100,8 @@ pub struct Process {
     written_bytes: u64,
 }
 
-impl ProcessExt for Process {
-    fn new(pid: Pid, parent: Option<Pid>, start_time: u64) -> Process {
+impl Process {
+    pub(crate) fn new(pid: Pid, parent: Option<Pid>, start_time: u64) -> Process {
         Process {
             name: String::with_capacity(20),
             pid,
@@ -123,6 +120,7 @@ impl ProcessExt for Process {
             old_stime: 0,
             updated: true,
             start_time,
+            run_time: 0,
             uid: 0,
             gid: 0,
             status: ProcessStatus::Unknown(0),
@@ -138,8 +136,14 @@ impl ProcessExt for Process {
             written_bytes: 0,
         }
     }
+}
 
-    fn kill(&self, signal: Signal) -> bool {
+impl ProcessExt for Process {
+    fn kill(&self) -> bool {
+        self.kill_with(Signal::Kill).unwrap()
+    }
+
+    fn kill_with(&self, signal: Signal) -> Option<bool> {
         let c_signal = match signal {
             Signal::Hangup => libc::SIGHUP,
             Signal::Interrupt => libc::SIGINT,
@@ -174,7 +178,7 @@ impl ProcessExt for Process {
             Signal::Power => libc::SIGPWR,
             Signal::Sys => libc::SIGSYS,
         };
-        unsafe { kill(self.pid, c_signal) == 0 }
+        unsafe { Some(kill(self.pid, c_signal) == 0) }
     }
 
     fn name(&self) -> &str {
@@ -225,15 +229,19 @@ impl ProcessExt for Process {
         self.start_time
     }
 
+    fn run_time(&self) -> u64 {
+        self.run_time
+    }
+
     fn cpu_usage(&self) -> f32 {
         self.cpu_usage
     }
 
     fn disk_usage(&self) -> DiskUsage {
         DiskUsage {
-            written_bytes: self.written_bytes - self.old_written_bytes,
+            written_bytes: self.written_bytes.saturating_sub(self.old_written_bytes),
             total_written_bytes: self.written_bytes,
-            read_bytes: self.read_bytes - self.old_read_bytes,
+            read_bytes: self.read_bytes.saturating_sub(self.old_read_bytes),
             total_read_bytes: self.read_bytes,
         }
     }
@@ -262,7 +270,6 @@ pub fn compute_cpu_usage(p: &mut Process, total_time: f32, max_value: f32) {
         / total_time
         * 100.)
         .min(max_value);
-    p.updated = false;
 }
 
 pub fn set_time(p: &mut Process, utime: u64, stime: u64) {
@@ -273,10 +280,6 @@ pub fn set_time(p: &mut Process, utime: u64, stime: u64) {
     p.updated = true;
 }
 
-pub fn has_been_updated(p: &Process) -> bool {
-    p.updated
-}
-
 pub(crate) fn update_process_disk_activity(p: &mut Process, path: &Path) {
     let mut path = PathBuf::from(path);
     path.push("io");
@@ -330,9 +333,10 @@ pub(crate) fn _get_process_data(
     page_size_kb: u64,
     pid: Pid,
     uptime: u64,
-    now: u64,
+    clock_cycle: u64,
+    refresh_kind: ProcessRefreshKind,
 ) -> Result<(Option<Process>, Pid), ()> {
-    let nb = match path.file_name().and_then(|x| x.to_str()).map(Pid::from_str) {
+    let pid = match path.file_name().and_then(|x| x.to_str()).map(Pid::from_str) {
         Some(Ok(nb)) if nb != pid => nb,
         _ => return Err(()),
     };
@@ -346,7 +350,7 @@ pub(crate) fn _get_process_data(
     };
     let parent_memory = proc_list.memory;
     let parent_virtual_memory = proc_list.virtual_memory;
-    if let Some(ref mut entry) = proc_list.tasks.get_mut(&nb) {
+    if let Some(ref mut entry) = proc_list.tasks.get_mut(&pid) {
         let data = if let Some(ref mut f) = entry.stat_file {
             get_all_data_from_file(f, 1024).map_err(|_| ())?
         } else {
@@ -366,12 +370,14 @@ pub(crate) fn _get_process_data(
             page_size_kb,
             parent_memory,
             parent_virtual_memory,
-            nb,
             uptime,
-            now,
+            clock_cycle,
+            refresh_kind,
         );
-        update_process_disk_activity(entry, path);
-        return Ok((None, nb));
+        if refresh_kind.disk_usage() {
+            update_process_disk_activity(entry, path);
+        }
+        return Ok((None, pid));
     }
 
     let mut tmp = PathBuf::from(path);
@@ -392,10 +398,8 @@ pub(crate) fn _get_process_data(
         }
     };
 
-    let clock_cycle = unsafe { sysconf(_SC_CLK_TCK) } as u64;
-    let since_boot = u64::from_str(parts[21]).unwrap_or(0) / clock_cycle;
-    let start_time = now.saturating_sub(uptime.saturating_sub(since_boot));
-    let mut p = Process::new(nb, parent_pid, start_time);
+    let start_time = u64::from_str(parts[21]).unwrap_or(0) / clock_cycle;
+    let mut p = Process::new(pid, parent_pid, start_time);
 
     p.stat_file = stat_file;
     get_status(&mut p, parts[2]);
@@ -455,12 +459,14 @@ pub(crate) fn _get_process_data(
         page_size_kb,
         proc_list.memory,
         proc_list.virtual_memory,
-        nb,
         uptime,
-        now,
+        clock_cycle,
+        refresh_kind,
     );
-    update_process_disk_activity(&mut p, path);
-    Ok((Some(p), nb))
+    if refresh_kind.disk_usage() {
+        update_process_disk_activity(&mut p, path);
+    }
+    Ok((Some(p), pid))
 }
 
 #[allow(clippy::too_many_arguments)]
@@ -471,9 +477,9 @@ fn update_time_and_memory(
     page_size_kb: u64,
     parent_memory: u64,
     parent_virtual_memory: u64,
-    pid: Pid,
     uptime: u64,
-    now: u64,
+    clock_cycle: u64,
+    refresh_kind: ProcessRefreshKind,
 ) {
     {
         // rss
@@ -491,8 +497,17 @@ fn update_time_and_memory(
             u64::from_str(parts[13]).unwrap_or(0),
             u64::from_str(parts[14]).unwrap_or(0),
         );
+        entry.run_time = uptime.saturating_sub(entry.start_time);
     }
-    refresh_procs(entry, &path.join("task"), page_size_kb, pid, uptime, now);
+    refresh_procs(
+        entry,
+        &path.join("task"),
+        page_size_kb,
+        entry.pid,
+        uptime,
+        clock_cycle,
+        refresh_kind,
+    );
 }
 
 pub(crate) fn refresh_procs(
@@ -501,7 +516,8 @@ pub(crate) fn refresh_procs(
     page_size_kb: u64,
     pid: Pid,
     uptime: u64,
-    now: u64,
+    clock_cycle: u64,
+    refresh_kind: ProcessRefreshKind,
 ) -> bool {
     if let Ok(d) = fs::read_dir(path) {
         let folders = d
@@ -533,7 +549,8 @@ pub(crate) fn refresh_procs(
                         page_size_kb,
                         pid,
                         uptime,
-                        now,
+                        clock_cycle,
+                        refresh_kind,
                     ) {
                         p
                     } else {
@@ -546,9 +563,15 @@ pub(crate) fn refresh_procs(
             let new_tasks = folders
                 .iter()
                 .filter_map(|e| {
-                    if let Ok((p, pid)) =
-                        _get_process_data(e.as_path(), proc_list, page_size_kb, pid, uptime, now)
-                    {
+                    if let Ok((p, pid)) = _get_process_data(
+                        e.as_path(),
+                        proc_list,
+                        page_size_kb,
+                        pid,
+                        uptime,
+                        clock_cycle,
+                        refresh_kind,
+                    ) {
                         updated_pids.push(pid);
                         p
                     } else {
