@@ -1,12 +1,8 @@
-//
-// Sysinfo
-//
-// Copyright (c) 2015 Guillaume Gomez
-//
+// Take a look at the license at the top of the repository in the LICENSE file.
 
 use crate::{
     sys::{component::Component, Disk, Networks, Process, Processor},
-    LoadAvg, Pid, RefreshKind, SystemExt, User,
+    LoadAvg, Pid, ProcessRefreshKind, RefreshKind, SystemExt, User,
 };
 
 use std::collections::HashMap;
@@ -35,9 +31,9 @@ impl SystemExt for System {
 
     fn refresh_components_list(&mut self) {}
 
-    fn refresh_processes(&mut self) {}
+    fn refresh_processes_specifics(&mut self, _refresh_kind: ProcessRefreshKind) {}
 
-    fn refresh_process(&mut self, _pid: Pid) -> bool {
+    fn refresh_process_specifics(&mut self, _pid: Pid, _refresh_kind: ProcessRefreshKind) -> bool {
         false
     }
 
