@@ -1,21 +1,15 @@
-//
-// Sysinfo
-//
-// Copyright (c) 2018 Guillaume Gomez
-//
+// Take a look at the license at the top of the repository in the LICENSE file.
 
-use crate::{LoadAvg, Networks, Pid, ProcessExt, RefreshKind, SystemExt, User};
+use crate::{LoadAvg, Networks, Pid, ProcessExt, ProcessRefreshKind, RefreshKind, SystemExt, User};
 use winapi::um::winreg::HKEY_LOCAL_MACHINE;
 
 use crate::sys::component::{self, Component};
 use crate::sys::disk::Disk;
-use crate::sys::process::{
-    compute_cpu_usage, get_handle, get_system_computation_time, update_disk_usage, update_memory,
-    Process,
-};
+use crate::sys::process::{get_handle, update_memory, Process};
 use crate::sys::processor::*;
 use crate::sys::tools::*;
 use crate::sys::users::get_users;
+use crate::sys::utils::get_now;
 
 use crate::utils::into_iter;
 
@@ -52,8 +46,7 @@ pub struct System {
     mem_available: u64,
     swap_total: u64,
     swap_used: u64,
-    global_processor: Processor,
-    processors: Vec<Processor>,
+    processors: ProcessorsWrapper,
     components: Vec<Component>,
     disks: Vec<Disk>,
     query: Option<Query>,
@@ -84,59 +77,48 @@ impl SystemExt for System {
 
     #[allow(non_snake_case)]
     fn new_with_specifics(refreshes: RefreshKind) -> System {
-        let (processors, vendor_id, brand) = init_processors();
         let mut s = System {
             process_list: HashMap::with_capacity(500),
             mem_total: 0,
             mem_available: 0,
             swap_total: 0,
             swap_used: 0,
-            global_processor: Processor::new_with_values("Total CPU", vendor_id, brand, 0),
-            processors,
+            processors: ProcessorsWrapper::new(),
             components: Vec::new(),
             disks: Vec::with_capacity(2),
-            query: Query::new(),
+            query: None,
             networks: Networks::new(),
             boot_time: unsafe { boot_time() },
             users: Vec::new(),
         };
-        // TODO: in case a translation fails, it might be nice to log it somewhere...
-        if let Some(ref mut query) = s.query {
-            let x = unsafe { load_symbols() };
-            if let Some(processor_trans) = get_translation(&"Processor".to_owned(), &x) {
-                // let idle_time_trans = get_translation(&"% Idle Time".to_owned(), &x);
-                let proc_time_trans = get_translation(&"% Processor Time".to_owned(), &x);
-                if let Some(ref proc_time_trans) = proc_time_trans {
-                    add_counter(
-                        format!("\\{}(_Total)\\{}", processor_trans, proc_time_trans),
-                        query,
-                        get_key_used(&mut s.global_processor),
-                        "tot_0".to_owned(),
-                    );
-                }
-                for (pos, proc_) in s.processors.iter_mut().enumerate() {
-                    if let Some(ref proc_time_trans) = proc_time_trans {
-                        add_counter(
-                            format!("\\{}({})\\{}", processor_trans, pos, proc_time_trans),
-                            query,
-                            get_key_used(proc_),
-                            format!("{}_0", pos),
-                        );
-                    }
-                }
-            } else {
-                sysinfo_debug!("failed to get `Processor` translation");
-            }
-        }
         s.refresh_specifics(refreshes);
         s
     }
 
     fn refresh_cpu(&mut self) {
+        if self.query.is_none() {
+            self.query = Query::new();
+            if let Some(ref mut query) = self.query {
+                add_english_counter(
+                    r"\Processor(_Total)\% Processor Time".to_string(),
+                    query,
+                    get_key_used(self.processors.global_processor_mut()),
+                    "tot_0".to_owned(),
+                );
+                for (pos, proc_) in self.processors.iter_mut().enumerate() {
+                    add_english_counter(
+                        format!(r"\Processor({})\% Processor Time", pos),
+                        query,
+                        get_key_used(proc_),
+                        format!("{}_0", pos),
+                    );
+                }
+            }
+        }
         if let Some(ref mut query) = self.query {
             query.refresh();
             let mut used_time = None;
-            if let Some(ref key_used) = *get_key_used(&mut self.global_processor) {
+            if let Some(ref key_used) = *get_key_used(self.processors.global_processor_mut()) {
                 used_time = Some(
                     query
                         .get(&key_used.unique_id)
@@ -144,7 +126,9 @@ impl SystemExt for System {
                 );
             }
             if let Some(used_time) = used_time {
-                self.global_processor.set_cpu_usage(used_time);
+                self.processors
+                    .global_processor_mut()
+                    .set_cpu_usage(used_time);
             }
             for p in self.processors.iter_mut() {
                 let mut used_time = None;
@@ -193,17 +177,17 @@ impl SystemExt for System {
     }
 
     #[allow(clippy::map_entry)]
-    fn refresh_process(&mut self, pid: Pid) -> bool {
+    fn refresh_process_specifics(&mut self, pid: Pid, refresh_kind: ProcessRefreshKind) -> bool {
         if self.process_list.contains_key(&pid) {
-            if !refresh_existing_process(self, pid) {
+            if !refresh_existing_process(self, pid, refresh_kind) {
                 self.process_list.remove(&pid);
                 return false;
             }
-            true
-        } else if let Some(mut p) = Process::new_from_pid(pid) {
-            let system_time = get_system_computation_time();
-            compute_cpu_usage(&mut p, self.processors.len() as u64, system_time);
-            update_disk_usage(&mut p);
+            return true;
+        }
+        let now = get_now();
+        if let Some(mut p) = Process::new_from_pid(pid, now) {
+            p.update(refresh_kind, self.processors.len() as u64, now);
             self.process_list.insert(pid, p);
             true
         } else {
@@ -212,9 +196,10 @@ impl SystemExt for System {
     }
 
     #[allow(clippy::cast_ptr_alignment)]
-    fn refresh_processes(&mut self) {
+    fn refresh_processes_specifics(&mut self, refresh_kind: ProcessRefreshKind) {
         // Windows 10 notebook requires at least 512KiB of memory to make it in one go
         let mut buffer_size: usize = 512 * 1024;
+        let now = get_now();
 
         loop {
             let mut process_information: Vec<u8> = Vec::with_capacity(buffer_size);
@@ -258,9 +243,12 @@ impl SystemExt for System {
 
                     process_information_offset += pi.NextEntryOffset as isize;
                 }
-                let nb_processors = self.processors.len() as u64;
                 let process_list = Wrap(UnsafeCell::new(&mut self.process_list));
-                let system_time = get_system_computation_time();
+                let nb_processors = if refresh_kind.cpu() {
+                    self.processors.len() as u64
+                } else {
+                    0
+                };
 
                 #[cfg(feature = "multithread")]
                 use rayon::iter::ParallelIterator;
@@ -274,9 +262,7 @@ impl SystemExt for System {
                         if let Some(proc_) = (*process_list.0.get()).get_mut(&pid) {
                             proc_.memory = (pi.WorkingSetSize as u64) / 1_000;
                             proc_.virtual_memory = (pi.VirtualSize as u64) / 1_000;
-                            compute_cpu_usage(proc_, nb_processors, system_time);
-                            update_disk_usage(proc_);
-                            proc_.updated = true;
+                            proc_.update(refresh_kind, nb_processors, now);
                             return None;
                         }
                         let name = get_process_name(&pi, pid);
@@ -290,9 +276,9 @@ impl SystemExt for System {
                             (pi.WorkingSetSize as u64) / 1_000,
                             (pi.VirtualSize as u64) / 1_000,
                             name,
+                            now,
                         );
-                        compute_cpu_usage(&mut p, nb_processors, system_time);
-                        update_disk_usage(&mut p);
+                        p.update(refresh_kind, nb_processors, now);
                         Some(p)
                     })
                     .collect::<Vec<_>>();
@@ -336,11 +322,11 @@ impl SystemExt for System {
     }
 
     fn global_processor_info(&self) -> &Processor {
-        &self.global_processor
+        self.processors.global_processor()
     }
 
     fn processors(&self) -> &[Processor] {
-        &self.processors
+        self.processors.processors()
     }
 
     fn physical_core_count(&self) -> Option<usize> {
@@ -473,18 +459,13 @@ fn is_proc_running(handle: HANDLE) -> bool {
     !(ret == FALSE || exit_code != STILL_ACTIVE)
 }
 
-fn refresh_existing_process(s: &mut System, pid: Pid) -> bool {
+fn refresh_existing_process(s: &mut System, pid: Pid, refresh_kind: ProcessRefreshKind) -> bool {
     if let Some(ref mut entry) = s.process_list.get_mut(&(pid as usize)) {
         if !is_proc_running(get_handle(entry)) {
             return false;
         }
         update_memory(entry);
-        update_disk_usage(entry);
-        compute_cpu_usage(
-            entry,
-            s.processors.len() as u64,
-            get_system_computation_time(),
-        );
+        entry.update(refresh_kind, s.processors.len() as u64, get_now());
         true
     } else {
         false
